@@ -3,8 +3,10 @@ The main class which you import and use a decorator.
 """
 import functools
 import json
+import logging
 import re
-from collections import defaultdict
+import sys
+import traceback
 from http import HTTPStatus
 from inspect import signature
 from typing import Iterable, Optional, Union
@@ -17,21 +19,11 @@ from pydantic_lambda_handler.middleware import BaseHook
 from pydantic_lambda_handler.models import BaseOutput
 
 
-def to_camel_case(text):
-    s = text.replace("-", " ").replace("_", " ")
-    s = s.split()
-    if len(text) == 0:
-        return text.capitalize()
-    return "".join(i.capitalize() for i in s)
-
-
 class PydanticLambdaHandler:
     """
     The decorator handle.
     """
 
-    cdk_stuff: dict = defaultdict(dict)
-    testing_stuff: dict = defaultdict(dict)
     _hooks: list[type[BaseHook]] = []
 
     def __init__(
@@ -54,6 +46,8 @@ class PydanticLambdaHandler:
         operation_id: str = None,
         description: str = "Successful Response",
         function_name=None,
+        response_model=None,
+        logger=None,
     ):
         """Expect request with a GET method.
 
@@ -62,7 +56,55 @@ class PydanticLambdaHandler:
         :return:
         """
         method = "get"
-        return self.run_method(method, url, status_code, operation_id, description, function_name)
+        self.logger = logger or logging.getLogger(__name__)
+        try:
+            return self.run_method(
+                method,
+                url,
+                status_code,
+                operation_id,
+                description,
+                function_name,
+                response_model,
+            )
+        except Exception as error:
+            traceback.print_exc(file=sys.stdout)
+            self.logger.error(f"{type(error).__name__}: {error}")
+            raise
+
+    def post(
+        self,
+        url,
+        *,
+        status_code: Union[HTTPStatus, int] = HTTPStatus.CREATED,
+        operation_id: str = None,
+        description: str = "Successful Response",
+        function_name=None,
+        response_model=None,
+        logger=None,
+    ):
+        """Expect request with a POST method.
+
+        :param url:
+        :param status_code:
+        :return:
+        """
+        method = "post"
+        self.logger = logger or logging.getLogger(__name__)
+        try:
+            return self.run_method(
+                method,
+                url,
+                status_code,
+                operation_id,
+                description,
+                function_name,
+                response_model,
+            )
+        except Exception as error:
+            traceback.print_exc(file=sys.stdout)
+            self.logger.error(f"{type(error).__name__}: {error}")
+            raise
 
     def run_method(
         self,
@@ -72,15 +114,10 @@ class PydanticLambdaHandler:
         operation_id,
         description,
         function_name,
+        response_model,
     ):
         for hook in self._hooks:
             hook.method_init(**locals())
-        ret_dict = add_resource(self.cdk_stuff, url.lstrip("/"))
-        testing_url = url.replace("{", "(?P<").replace("}", r">\w+)")
-        if testing_url not in self.testing_stuff["paths"]:
-            self.testing_stuff["paths"][testing_url] = {method: {}}
-        else:
-            self.testing_stuff["paths"][testing_url][method] = {}
 
         def create_response(func):
             for hook in self._hooks:
@@ -89,7 +126,7 @@ class PydanticLambdaHandler:
             sig = signature(func)
 
             if sig.parameters:
-                EventModel = self.generate_get_event_model(url, sig)
+                EventModel = self.generate_event_model(url, sig)
 
             @functools.wraps(func)
             def wrapper_decorator(event, context: LambdaContext):
@@ -101,6 +138,7 @@ class PydanticLambdaHandler:
                 if sig.parameters:
                     path_parameters = event.get("pathParameters", {}) or {}
                     query_parameters = event.get("queryStringParameters", {}) or {}
+
                     if event["body"] is not None:
                         body = loads(event["body"])
                     else:
@@ -119,6 +157,11 @@ class PydanticLambdaHandler:
                     func_kwargs = {**event_model.path.dict(), **event_model.query.dict()}
                     if hasattr(event_model, "body"):
                         func_kwargs.update(**{event_model.body._alias: event_model.body})
+
+                    if context_name := next(
+                        (i.name for i in iter(sig.parameters.values()) if issubclass(i.annotation, LambdaContext)), None
+                    ):
+                        func_kwargs[context_name] = context
                     body = func(**func_kwargs)
                 else:
                     body = func()
@@ -127,28 +170,29 @@ class PydanticLambdaHandler:
                     body = hook.post_func(body)
 
                 if hasattr(body, "json"):
-                    body = loads(body.json())
+                    response = BaseOutput(body=body.json(), status_code=status_code)
+                else:
+                    response = BaseOutput(body=json.dumps(body), status_code=status_code)
 
-                response = BaseOutput(body=json.dumps(body), status_code=status_code)
                 return loads(response.json())
 
-            add_methods(method, func, ret_dict, function_name, str(int(status_code)))
-
-            self.testing_stuff["paths"][testing_url][method]["handler"]["function"] = func
-
             return wrapper_decorator
 
-        self.testing_stuff["paths"][testing_url][method]["handler"] = {"decorated_function": create_response}
+        for hook in self._hooks:
+            hook.post_create_response(**locals())
         return create_response
 
     @staticmethod
-    def generate_get_event_model(url, sig):
+    def generate_event_model(url, sig):
         path_model_dict = {}
         query_model_dict = {}
+
         body_default = None
         body_model = None
+
         path_parameters_list = list(re.findall(r"\{(.*?)\}", url))
         path_parameters = set(path_parameters_list)
+        additional_kwargs = {}
         if len(path_parameters_list) != len(path_parameters):
             raise ValueError(f"re-declared path variable: {url}")
         for param, param_info in sig.parameters.items():
@@ -175,6 +219,8 @@ class PydanticLambdaHandler:
                         raise ValueError("Can only use one Pydantic model for body only")
                     body_model = model
                     body_model._alias = param
+                elif issubclass(model, LambdaContext):
+                    additional_kwargs[param] = annotations
                 else:
                     query_model_dict[param] = annotations
 
@@ -188,112 +234,3 @@ class PydanticLambdaHandler:
             event_models["body"] = (body_model, body_default)
 
         return create_model("EventModel", **event_models)
-
-    @staticmethod
-    def generate_post_event_model(url, sig):
-        path_model_dict = {}
-        query_model_dict = {}
-
-        body_default = None
-        body_model = None
-
-        path_parameters_list = list(re.findall(r"\{(.*?)\}", url))
-        path_parameters = set(path_parameters_list)
-
-        if len(path_parameters_list) != len(path_parameters):
-            raise ValueError(f"re-declared path variable: {url}")
-
-        for param, param_info in sig.parameters.items():
-            if param in path_parameters:
-                if param_info.annotation == param_info.empty:
-                    annotations = str, ...
-                else:
-                    annotations = param_info.annotation, ...
-            else:
-                default = ... if param_info.default == param_info.empty else param_info.default
-                if param_info.annotation == param_info.empty:
-                    annotations = str, default
-                else:
-                    annotations = param_info.annotation, default
-
-            if param in path_parameters:
-                if param_info.default != param_info.empty:
-                    raise ValueError("Should not set default for path parameters")
-                path_model_dict[param] = annotations
-            else:
-                model, body_default = annotations
-                if issubclass(model, BaseModel):
-                    if body_model:
-                        raise ValueError("Can only use one Pydantic model for body only")
-                    body_model = model
-                    body_model._alias = param
-                else:
-                    query_model_dict[param] = annotations
-
-        if path_parameters != set(path_model_dict.keys()):
-            raise ValueError("Missing path parameters")
-
-        PathModel = create_model("PathModel", **path_model_dict)
-        QueryModel = create_model("QueryModel", **query_model_dict)
-
-        return create_model(
-            "EventModel", **{"path": (PathModel, {}), "query": (QueryModel, {}), "body": (body_model, body_default)}
-        )
-
-    def post(
-        self,
-        url,
-        *,
-        status_code: Union[HTTPStatus, int] = HTTPStatus.CREATED,
-        operation_id: str = None,
-        description: str = "Successful Response",
-        function_name=None,
-    ):
-        """Expect request with a POST method.
-
-        :param url:
-        :param status_code:
-        :return:
-        """
-        method = "post"
-        return self.run_method(
-            method,
-            url,
-            status_code,
-            operation_id,
-            description,
-            function_name,
-        )
-
-
-def add_methods(method, func, ret_dict, function_name, open_api_status_code):
-    if "methods" in ret_dict:
-        ret_dict["methods"][method] = {
-            "reference": f"{func.__module__}.{func.__qualname__}",
-            "status_code": open_api_status_code,
-            "function_name": function_name or to_camel_case(func.__name__),
-        }
-    else:
-        ret_dict["methods"] = {
-            method: {
-                "reference": f"{func.__module__}.{func.__qualname__}",
-                "status_code": open_api_status_code,
-                "function_name": function_name or to_camel_case(func.__name__),
-            }
-        }
-
-
-def add_resource(child_dict: dict, url):
-    part, found, remaining = url.partition("/")
-    if part:
-        if part in child_dict.get("resources", {}):
-            return add_resource(child_dict["resources"][part], remaining)
-
-        last_resource: dict[str, dict] = {}
-        if "resources" not in child_dict:
-            child_dict["resources"] = {part: last_resource}
-        else:
-            child_dict["resources"].update({part: last_resource})
-
-        return add_resource(child_dict["resources"][part], remaining)
-    return child_dict
