@@ -1,3 +1,4 @@
+import pytest
 from demo_app_handlers import create_handler  # type: ignore
 
 
@@ -60,3 +61,13 @@ def test_list_response_model(requests_client, base_url):
     response = requests_client.get(f"{base_url}/list_response_model")
     assert response.status_code == 200
     assert response.json() == [{"item_name": "secret"}]
+
+
+def test_error_much_handler(requests_client, base_url):
+    with pytest.raises(ValueError):
+        requests_client.get(f"{base_url}/error_much")
+
+
+def test_error_much_handler_post(requests_client, base_url):
+    with pytest.raises(ValueError):
+        requests_client.post(f"{base_url}/error_much")
